/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.framework.interceptor.description;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.objectweb.petals.component.framework.ExtensionsConstants;
import org.objectweb.petals.component.framework.PEtALSCDKException;
import org.objectweb.petals.component.framework.interceptor.Interceptor;
import org.objectweb.petals.component.framework.util.Extensions;
import org.objectweb.petals.component.framework.util.XMLUtil;
import org.objectweb.petals.jbi.descriptor.JBIDescriptor;
import org.objectweb.petals.jbi.descriptor.JBIDescriptorBuilder;
import org.objectweb.petals.jbi.descriptor.JBIDescriptorException;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 * 
 */
public final class InterceptorBuilder {

    public static final String DELIM = ",";

    public static final String ACCEPT = "accept";

    public static final String ACCEPT_RESPONSE = "accept-response";

    public static final String SEND = "send";

    public static final String SEND_RESPONSE = "send-response";

    public static final String INTERCEPTOR_NODE = "interceptor";

    public static final String PARAMETER_NODE = "param";

    public static final String ORDER_NODE = "order";

    public static final String CLASS_ATTR = "class";

    public static final String ACTIVE_ATTR = "active";

    public static final String MODE_ATTR = "mode";

    public static final String NAME_ATTR = "name";

    public static final String VALUE_ATTR = "value";

    private static InterceptorBuilder instance = null;

    /**
     * Creates a new instance of {@link InterceptorBuilder}
     * 
     */
    private InterceptorBuilder() {
    }

    /**
     * Get loader instance
     * 
     * @return
     */
    public synchronized static final InterceptorBuilder getInstance() {
        if (InterceptorBuilder.instance == null) {
            InterceptorBuilder.instance = new InterceptorBuilder();
        }
        return InterceptorBuilder.instance;
    }

    /**
     * Load the component interceptors.
     * 
     * @param rootPath
     */
    public List<InterceptorConfiguration> loadComponentInterceptors(final String rootPath)
            throws PEtALSCDKException {
        final File jbiFile = new File(rootPath, JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE);
        JBIDescriptor descriptor = null;

        try {
            final JBIDescriptorBuilder builder = JBIDescriptorBuilder.getInstance();
            descriptor = builder.buildDescriptor(jbiFile.toURI());
        } catch (final JBIDescriptorException e) {
            throw new PEtALSCDKException("Can not load the JBI descriptor file");
        }

        final org.objectweb.petals.jbi.descriptor.Extensions descExtensions = descriptor
                .getComponent().getExtensions();
        if (descExtensions == null) {
            return null;
        }
        final Extensions extensions = new Extensions(descExtensions);

        return this.loadInterceptors(extensions);
    }

    /**
     * 
     * @param extensions
     * @return
     */
    public List<InterceptorConfiguration> loadInterceptors(final Extensions extensions) {
        final List<InterceptorConfiguration> interceptors = new ArrayList<InterceptorConfiguration>();

        final Node interceptorNode = XMLUtil.findChild(extensions.getDocumentFragment(),
                ExtensionsConstants.COMPONENT_INTERCEPTORS, Extensions.PETALS_EXTENSIONS_URI, true);
        if (interceptorNode != null) {
            final List<Node> children = XMLUtil.getNodeChildren(interceptorNode);
            for (final Node childNode : children) {
                final InterceptorConfiguration conf = this.loadInterceptorDescriptor(childNode);
                if (conf != null) {
                    interceptors.add(conf);
                }
            }
        }
        return interceptors;
    }

    /**
     * Load the interceptor configuration from the extensions
     * 
     * @param extensions
     * @return interceptor configuration defined in the SU or null if no
     *         interceptor extension has been defined in the extensions.
     */
    public ExtensibleElementInterceptorConfiguration loadSUInterceptor(final Extensions extensions) {
        final ExtensibleElementInterceptorConfiguration result = new ExtensibleElementInterceptorConfiguration();

        if ((extensions != null) && (extensions.getDocumentFragment() != null)) {
            final Node interceptorsNode = XMLUtil.findChild(extensions.getDocumentFragment(),
                    ExtensionsConstants.SU_INTERCEPTORS, Extensions.PETALS_EXTENSIONS_URI, true);

            if (interceptorsNode != null) {

                final Node inNode = XMLUtil.findChild(interceptorsNode, InterceptorBuilder.ACCEPT,
                        Extensions.PETALS_EXTENSIONS_URI, true);
                final Node outNode = XMLUtil.findChild(interceptorsNode, InterceptorBuilder.SEND,
                        Extensions.PETALS_EXTENSIONS_URI, true);
                final Node inResponseNode = XMLUtil.findChild(interceptorsNode,
                        InterceptorBuilder.ACCEPT_RESPONSE, Extensions.PETALS_EXTENSIONS_URI, true);
                final Node outResponseNode = XMLUtil.findChild(interceptorsNode,
                        InterceptorBuilder.SEND_RESPONSE, Extensions.PETALS_EXTENSIONS_URI, true);

                // set interceptors in configuration
                result.setInputInterceptors(this.getOrderedInterceptors(inNode));
                result.setOutputInterceptors(this.getOrderedInterceptors(outNode));
                result.setInputResponseInterceptors(this.getOrderedInterceptors(inResponseNode));
                result.setOutputResponseInterceptors(this.getOrderedInterceptors(outResponseNode));

            }
        }
        return result;
    }

    /**
     * 
     * @param orderNode
     * @return
     */
    private Map<String, InterceptorConfiguration> getOrderedInterceptors(final Node orderNode) {
        final Map<String, InterceptorConfiguration> result = new LinkedHashMap<String, InterceptorConfiguration>();
        if ((orderNode != null)
                && (orderNode.getNamespaceURI() != null)
                && orderNode.getNamespaceURI().equals(Extensions.PETALS_EXTENSIONS_URI)
                && (orderNode.getLocalName().equals(InterceptorBuilder.ACCEPT)
                        || orderNode.getLocalName().equals(InterceptorBuilder.ACCEPT_RESPONSE)
                        || orderNode.getLocalName().equals(InterceptorBuilder.SEND) || orderNode
                        .getLocalName().equals(InterceptorBuilder.SEND_RESPONSE))) {

            final List<Node> nodes = XMLUtil.getNodeChildren(orderNode);
            for (final Node node : nodes) {
                final InterceptorConfiguration config = this.loadInterceptorDescriptor(node);
                result.put(config.getName(), config);
            }
        }
        return result;
    }

    /**
     * Load the interceptor descriptor from the given node.
     * 
     * @param interceptorNode
     *            the {@link Node} to create interceptor description from.
     * @return an {@link InterceptorConfiguration} or null if it is not an
     *         interceptor node or if the node is null.
     */
    public InterceptorConfiguration loadInterceptorDescriptor(final Node interceptorNode) {
        InterceptorConfiguration result = null;

        if ((interceptorNode != null)
                && interceptorNode.getNamespaceURI().equals(Extensions.PETALS_EXTENSIONS_URI)
                && interceptorNode.getLocalName().equals(InterceptorBuilder.INTERCEPTOR_NODE)) {
            result = new InterceptorConfiguration();
            final NamedNodeMap atts = interceptorNode.getAttributes();
            if (atts == null) {
                return result;
            }

            // get the name, class and active attributes
            final Node itcNameNode = atts.getNamedItem(InterceptorBuilder.NAME_ATTR);
            final Node itcClassNode = atts.getNamedItem(InterceptorBuilder.CLASS_ATTR);
            final Node itcActiveNode = atts.getNamedItem(InterceptorBuilder.ACTIVE_ATTR);
            if (itcNameNode != null) {
                result.setName(itcNameNode.getNodeValue());
            }
            if (itcClassNode != null) {
                result.setClazz(itcClassNode.getNodeValue());
            }
            if ((itcActiveNode != null) && itcActiveNode.getNodeValue().equals("true")) {
                result.setActive(true);
            }

            // get the parameters
            Map<String, String> parameters = new HashMap<String, String>();
            final NodeList paramNodes = interceptorNode.getChildNodes();
            for (int i = 0; i < paramNodes.getLength(); i++) {
                final Node n = paramNodes.item(i);
                if ((n != null) && (n.getNamespaceURI() != null)
                        && n.getNamespaceURI().equals(Extensions.PETALS_EXTENSIONS_URI)
                        && n.getLocalName().equals(Extensions.PETALS_PARAMETERS_NAME)) {
                    parameters = Extensions.getPetalsParameters(n);
                }
            }
            result.setParameters(parameters);
        }
        return result;
    }

    /**
     * Load an interceptor instance
     * 
     * @return
     */
    public Interceptor loadInstance(final String className) throws PEtALSCDKException {
        Interceptor interceptor = null;
        final Class clazz = this.loadClass(className);

        try {
            final Object o = clazz.newInstance();
            if (o instanceof Interceptor) {
                interceptor = (Interceptor) o;
            } else {
                throw new PEtALSCDKException("Invalid class, " + className
                        + " is not an interceptor");
            }

        } catch (final InstantiationException e) {
            throw new PEtALSCDKException("Can not instanciate interceptor", e);
        } catch (final IllegalAccessException e) {
            throw new PEtALSCDKException("Illegal access to interceptor", e);
        }

        return interceptor;
    }

    /**
     * 
     * @param classloader
     * @param className
     * @return
     * @throws JBIException
     */
    protected Class loadClass(final String className) throws PEtALSCDKException {
        Class clazz;
        final ClassLoader classloader = this.getClass().getClassLoader();
        try {
            clazz = classloader.loadClass(className);
            if (clazz == null) {
                throw new PEtALSCDKException("Could not find interceptor class : " + className);
            }

        } catch (final ClassNotFoundException e) {
            throw new PEtALSCDKException("Class not found interceptor class : " + className, e);
        } catch (final NullPointerException e) {
            throw new PEtALSCDKException("Null class : " + className);
        }
        return clazz;
    }
}
