/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SourceHelper.java 94 2006-03-26 17:11:05Z alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.framework.util;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Properties;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.objectweb.petals.component.framework.PEtALSCDKException;

/**
 * Sample class used to manage the mapping between
 * <code>javax.xml.transform.Source</code> from String. the mapping is a
 * quicly "string" -> "<content>string</content>" conversion. Initial
 * developer(s): Adrien LOUIS
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 */
public final class SourceHelper {

    private static final String START_TAG = "<content>";

    private static final String END_TAG = "</content>";

    private static final int SOAP_FAULT_BUFFER_SIZE = 2048;
    
    private static final String UTF8_CHARFORMAT="UTF-8";

    private static Transformer transformer;

    /**
     * Creates a new instance of {@link SourceHelper}
     * 
     */
    private SourceHelper() {
        super();
    }

    /**
     * Create a Source from String adding start and end tags.
     * 
     * @param msg
     * @return
     * @throws PEtALSCDKException
     */
    public static Source createContentSource(final String msg)
            throws PEtALSCDKException {
        return createSource(START_TAG + msg + END_TAG);
    }
    
    /**
     * Creates an XML fault with the stacktrace, message and the specified code
     * 
     * @param e
     *            exception, used to provide faultstring and detail
     * @param code
     *            code of the fault
     * @return
     */
    public static String createSoapFault(final Throwable e, final String code) {
        StringBuffer s = new StringBuffer(SOAP_FAULT_BUFFER_SIZE);
        s
                .append("<s:Fault xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\">"
                        + "<s:faultcode>");
        if (code != null) {
            s.append(code);
        }
        s.append("</s:faultcode><s:faultstring>");
        s.append(e.getMessage());
        s.append("</s:faultstring><s:detail><![CDATA[");
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        e.printStackTrace(pw);
        s.append(sw);
        s.append("]]></s:detail></s:Fault>");
        return s.toString();
    }

    /**
     * Create a Source from the String. The string must be in UTF-8 format,
     * otherwise, the Source is not created.
     * 
     * @param msg
     *            an UTF-8 String
     * @return the Source built from the String, null if not an UTF-8
     */
    public static Source createSource(final String msg)
            throws PEtALSCDKException {
        StreamSource source = new StreamSource();
        try {
            byte[] msgByte = msg.getBytes(UTF8_CHARFORMAT);
            ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
            source.setInputStream(in);
        } catch (IOException e) {
            throw new PEtALSCDKException(e);
        }
        return source;
    }

    /**
     * Create a Source from the String with a specified encoding format. If
     * charFormat is null or empty, UTF-8 will be used
     * 
     * @param msg
     *            The string to create source from
     * @param charFormat
     *            the encodig format
     * @return the Source built from the String, null if not an UTF-8
     */
    public static Source createSource(final String msg, final String charFormat)
            throws PEtALSCDKException {
        StreamSource source = new StreamSource();
        try {
            byte[] msgByte = msg.getBytes(StringHelper
                    .isNullOrEmpty(charFormat) ? UTF8_CHARFORMAT : charFormat);
            ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
            source.setInputStream(in);
        } catch (IOException e) {
            throw new PEtALSCDKException(e);
        }
        return source;
    }

    /**
     * Create a Source from String adding start and end tags.
     * 
     * @param msg
     * @return
     * @throws PEtALSCDKException
     */
    public static Source createContentSourceWithURLEncoding(final String msg,final String charFormat)
            throws PEtALSCDKException {
            Source source;
            try {
                String encodedMsg = URLEncoder.encode(msg, StringHelper
                        .isNullOrEmpty(charFormat) ? "UTF-8" : charFormat);
                source =createSource(START_TAG+encodedMsg+END_TAG, charFormat);
            } catch (UnsupportedEncodingException e) {
                throw new PEtALSCDKException(e);
            }

            return source;
            
    }
    
    /**
     * URL Encode a string then create a Source from it with a specified
     * encoding format. If charFormat is null or empty, UTF-8 will be used
     * 
     * @param msg
     *            The string to create source from
     * @param charFormat
     *            the encodig format
     * @return the Source built from the String, null if not an UTF-8
     */
    public static Source createSourceWithURLEncoding(final String msg,
            final String charFormat) throws PEtALSCDKException {
        Source source;
        try {
            String encodedMsg = URLEncoder.encode(msg, StringHelper
                    .isNullOrEmpty(charFormat) ? "UTF-8" : charFormat);
            source =createSource(encodedMsg, charFormat);
        } catch (UnsupportedEncodingException e) {
            throw new PEtALSCDKException(e);
        }

        return source;
    }

    /**
     * Create a String from a Source. The String will be in UTF-8.
     * 
     * @param s
     * @return
     * @throws PEtALSCDKException
     */
    public static String createString(final Source s) throws PEtALSCDKException {

        String result = null;
        try {
            if (s instanceof StreamSource
                    && ((StreamSource) s).getInputStream().markSupported()) {
                StreamSource ss = (StreamSource) s;
                ss.getInputStream().reset();
                byte[] buf = new byte[ss.getInputStream().available()];
                BufferedInputStream bis = new BufferedInputStream(ss
                        .getInputStream());
                bis.read(buf);
                result = new String(buf);
                ss.getInputStream().reset();
            } else {
                StringWriter buffer = new StringWriter();
                Result sresult = new StreamResult(buffer);
                Transformer transform = getTransformer();
                transform.transform(s, sresult);
                result = buffer.toString();
            }
        } catch (IOException e) {
            throw new PEtALSCDKException(e);
        } catch (TransformerConfigurationException e) {
            throw new PEtALSCDKException(e);
        } catch (TransformerException e) {
            throw new PEtALSCDKException(e);
        }
        return result;
    }

    /**
     * Create an String from a Source which has been url encoded. The String
     * will be in UTF-8 if charFormat is null or empty.
     * 
     * @param s
     * @return
     * @throws PEtALSCDKException
     */
    public static String createURLDecodedString(final Source s,
            final String charFormat) throws PEtALSCDKException {

        String result = null;
        try {
            result = createString(s, charFormat);
            result = URLDecoder.decode(result, StringHelper
                    .isNullOrEmpty(charFormat) ? UTF8_CHARFORMAT : charFormat);
        } catch (UnsupportedEncodingException e) {
            throw new PEtALSCDKException(e);
        }

        return result;
    }

    /**
     * Create a String from a Source with a given char encoding. The String will
     * be in UTF-8 if charFormat is null or empty.
     * 
     * @param s
     *            the source
     * @param charFormat
     *            the char encoding. e.g: UTF-8
     * @return
     * @throws PEtALSCDKException
     */
    public static String createString(final Source s, String charFormat)
            throws PEtALSCDKException {

        String result = null;
        try {
            if (s instanceof StreamSource
                    && ((StreamSource) s).getInputStream().markSupported()) {
                StreamSource ss = (StreamSource) s;
                ss.getInputStream().reset();
                byte[] buf = new byte[ss.getInputStream().available()];
                BufferedInputStream bis = new BufferedInputStream(ss
                        .getInputStream());
                bis.read(buf);
                result = new String(buf,
                        StringHelper.isNullOrEmpty(charFormat) ? UTF8_CHARFORMAT
                                : charFormat);
                ss.getInputStream().reset();
            } else {
                StringWriter buffer = new StringWriter();
                Result sresult = new StreamResult(buffer);
                Transformer transform = getTransformer();
                transform.transform(s, sresult);
                result = buffer.toString();
            }
        } catch (IOException e) {
            throw new PEtALSCDKException(e);
        } catch (TransformerConfigurationException e) {
            throw new PEtALSCDKException(e);
        } catch (TransformerException e) {
            throw new PEtALSCDKException(e);
        }
        return result;
    }

    /**
     * 
     * @return
     * @throws TransformerConfigurationException
     */
    protected synchronized static Transformer getTransformer()
            throws TransformerConfigurationException {
        if (transformer == null) {
            transformer = TransformerFactory.newInstance().newTransformer();
            Properties props = new Properties();
            props.put(OutputKeys.OMIT_XML_DECLARATION, "yes");
            transformer.setOutputProperties(props);
        }
        return transformer;
    }

    /**
     * Create a String from a Source.
     * 
     * @param s
     * @return the String in UTF-8 representation without the start and end tags
     * @throws PEtALSCDKException
     */
    public static String createStringContent(final Source s)
            throws PEtALSCDKException {
        String content = createString(s);
        int index = content.indexOf(START_TAG);
        String result = content.substring(index + START_TAG.length());
        return result.substring(0, result.indexOf(END_TAG));
    }

}
